@@ -2,12 +2,13 @@ use std::borrow::Cow;
 use std::fmt;
 
 use ahash::AHashMap;
+use num_bigint::BigInt;
 use pyo3::exceptions::{PyKeyError, PyTypeError, PyValueError};
 use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList};
 
-use crate::build_tools::{py_err, py_error_type};
+use crate::tools::{extract_i64, py_err, py_error_type};
 use strum::{Display, EnumMessage, IntoEnumIterator};
 use strum_macros::EnumIter;
 
@@ -174,6 +175,7 @@ pub enum ErrorType {
     // int errors
     IntType,
     IntParsing,
+    IntParsingSize,
     IntFromFloat,
     // ---------------------
     // float errors
@@ -234,8 +236,14 @@ pub enum ErrorType {
     },
     DatetimePast,
     DatetimeFuture,
-    DatetimeAware,
-    DatetimeNaive,
+    // ---------------------
+    // timezone errors
+    TimezoneNaive,
+    TimezoneAware,
+    TimezoneOffset {
+        tz_expected: i32,
+        tz_actual: i32,
+    },
     // ---------------------
     // timedelta errors
     TimeDeltaType,
@@ -416,6 +424,9 @@ impl ErrorType {
             Self::TimeParsing { .. } => extract_context!(Cow::Owned, TimeParsing, ctx, error: String),
             Self::DatetimeParsing { .. } => extract_context!(Cow::Owned, DatetimeParsing, ctx, error: String),
             Self::DatetimeObjectInvalid { .. } => extract_context!(DatetimeObjectInvalid, ctx, error: String),
+            Self::TimezoneOffset { .. } => {
+                extract_context!(TimezoneOffset, ctx, tz_expected: i32, tz_actual: i32)
+            }
             Self::TimeDeltaParsing { .. } => extract_context!(Cow::Owned, TimeDeltaParsing, ctx, error: String),
             Self::IsInstanceOf { .. } => extract_context!(IsInstanceOf, ctx, class: String),
             Self::IsSubclassOf { .. } => extract_context!(IsSubclassOf, ctx, class: String),
@@ -488,6 +499,7 @@ impl ErrorType {
             Self::IntType => "Input should be a valid integer",
             Self::IntParsing => "Input should be a valid integer, unable to parse string as an integer",
             Self::IntFromFloat => "Input should be a valid integer, got a number with a fractional part",
+            Self::IntParsingSize => "Unable to parse input string as an integer, exceeded maximum size",
             Self::FloatType => "Input should be a valid number",
             Self::FloatParsing => "Input should be a valid number, unable to parse string as an number",
             Self::BytesType => "Input should be a valid bytes",
@@ -508,10 +520,11 @@ impl ErrorType {
             Self::DatetimeType => "Input should be a valid datetime",
             Self::DatetimeParsing {..} => "Input should be a valid datetime, {error}",
             Self::DatetimeObjectInvalid {..} => "Invalid datetime object, got {error}",
-            Self::DatetimePast => "Datetime should be in the past",
-            Self::DatetimeFuture => "Datetime should be in the future",
-            Self::DatetimeAware => "Datetime should have timezone info",
-            Self::DatetimeNaive => "Datetime should not have timezone info",
+            Self::DatetimePast => "Input should be in the past",
+            Self::DatetimeFuture => "Input should be in the future",
+            Self::TimezoneNaive => "Input should not have timezone info",
+            Self::TimezoneAware => "Input should have timezone info",
+            Self::TimezoneOffset {..} => "Timezone offset of {tz_expected} required, got {tz_actual}",
             Self::TimeDeltaType => "Input should be a valid timedelta",
             Self::TimeDeltaParsing {..} => "Input should be a valid timedelta, {error}",
             Self::FrozenSetType => "Input should be a valid frozenset",
@@ -539,8 +552,13 @@ impl ErrorType {
     pub fn message_template_json(&self) -> &'static str {
         match self {
             Self::NoneRequired => "Input should be null",
-            Self::ListType => "Input should be a valid array",
-            Self::DataclassType { .. } => "Input should be an object",
+            Self::ListType | Self::TupleType | Self::IterableType | Self::SetType | Self::FrozenSetType => {
+                "Input should be a valid array"
+            }
+            Self::DictAttributesType | Self::DictType | Self::DataclassType { .. } => "Input should be an object",
+            Self::TimeDeltaType => "Input should be a valid duration",
+            Self::TimeDeltaParsing { .. } => "Input should be a valid duration, {error}",
+            Self::ArgumentsType => "Arguments must be an array or an object",
             _ => self.message_template_python(),
         }
     }
@@ -618,6 +636,7 @@ impl ErrorType {
             Self::TimeParsing { error } => render!(tmpl, error),
             Self::DatetimeParsing { error } => render!(tmpl, error),
             Self::DatetimeObjectInvalid { error } => render!(tmpl, error),
+            Self::TimezoneOffset { tz_expected, tz_actual } => to_string_render!(tmpl, tz_expected, tz_actual),
             Self::TimeDeltaParsing { error } => render!(tmpl, error),
             Self::IsInstanceOf { class } => render!(tmpl, class),
             Self::IsSubclassOf { class } => render!(tmpl, class),
@@ -675,6 +694,7 @@ impl ErrorType {
             Self::TimeParsing { error } => py_dict!(py, error),
             Self::DatetimeParsing { error } => py_dict!(py, error),
             Self::DatetimeObjectInvalid { error } => py_dict!(py, error),
+            Self::TimezoneOffset { tz_expected, tz_actual } => py_dict!(py, tz_expected, tz_actual),
             Self::TimeDeltaParsing { error } => py_dict!(py, error),
             Self::IsInstanceOf { class } => py_dict!(py, class),
             Self::IsSubclassOf { class } => py_dict!(py, class),
@@ -697,6 +717,7 @@ impl ErrorType {
 #[derive(Clone, Debug)]
 pub enum Number {
     Int(i64),
+    BigInt(BigInt),
     Float(f64),
     String(String),
 }
@@ -713,6 +734,12 @@ impl From<i64> for Number {
     }
 }
 
+impl From<BigInt> for Number {
+    fn from(i: BigInt) -> Self {
+        Self::BigInt(i)
+    }
+}
+
 impl From<f64> for Number {
     fn from(f: f64) -> Self {
         Self::Float(f)
@@ -727,7 +754,7 @@ impl From<String> for Number {
 
 impl FromPyObject<'_> for Number {
     fn extract(obj: &PyAny) -> PyResult<Self> {
-        if let Ok(int) = obj.extract::<i64>() {
+        if let Ok(int) = extract_i64(obj) {
             Ok(Number::Int(int))
         } else if let Ok(float) = obj.extract::<f64>() {
             Ok(Number::Float(float))
@@ -744,6 +771,7 @@ impl fmt::Display for Number {
         match self {
             Self::Float(s) => write!(f, "{s}"),
             Self::Int(i) => write!(f, "{i}"),
+            Self::BigInt(i) => write!(f, "{i}"),
             Self::String(s) => write!(f, "{s}"),
         }
     }
@@ -752,6 +780,7 @@ impl ToPyObject for Number {
     fn to_object(&self, py: Python<'_>) -> PyObject {
         match self {
             Self::Int(i) => i.into_py(py),
+            Self::BigInt(i) => i.clone().into_py(py),
             Self::Float(f) => f.into_py(py),
             Self::String(s) => s.into_py(py),
         }
