@@ -1,7 +1,6 @@
 use std::borrow::Cow;
 use std::str::from_utf8;
 
-use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::{
     PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFrozenSet, PyInt, PyIterator, PyList,
@@ -9,10 +8,10 @@ use pyo3::types::{
 };
 #[cfg(not(PyPy))]
 use pyo3::types::{PyDictItems, PyDictKeys, PyDictValues};
-use pyo3::{ffi, intern, AsPyPointer, PyTypeInfo};
+use pyo3::{intern, AsPyPointer, PyTypeInfo};
 
-use crate::build_tools::safe_repr;
 use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValResult};
+use crate::tools::{extract_i64, safe_repr};
 use crate::{ArgsKwargs, PyMultiHostUrl, PyUrl};
 
 use super::datetime::{
@@ -22,8 +21,8 @@ use super::datetime::{
 };
 use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
 use super::{
-    py_string_str, EitherBytes, EitherString, EitherTimedelta, GenericArguments, GenericIterable, GenericIterator,
-    GenericMapping, Input, JsonInput, PyArgs,
+    py_string_str, EitherBytes, EitherInt, EitherString, EitherTimedelta, GenericArguments, GenericIterable,
+    GenericIterator, GenericMapping, Input, JsonInput, PyArgs,
 };
 
 #[cfg(not(PyPy))]
@@ -90,7 +89,7 @@ impl<'a> Input<'a> for PyAny {
     fn as_loc_item(&self) -> LocItem {
         if let Ok(py_str) = self.downcast::<PyString>() {
             py_str.to_string_lossy().as_ref().into()
-        } else if let Ok(key_int) = self.extract::<i64>() {
+        } else if let Ok(key_int) = extract_i64(self) {
             key_int.into()
         } else {
             safe_repr(self).to_string().into()
@@ -109,12 +108,12 @@ impl<'a> Input<'a> for PyAny {
         self.is_none()
     }
 
-    fn input_get_attr(&self, name: &PyString) -> Option<PyResult<&PyAny>> {
-        Some(self.getattr(name))
-    }
-
-    fn is_exact_instance(&self, class: &PyType) -> bool {
-        self.get_type().is(class)
+    fn input_is_instance(&self, class: &PyType) -> Option<&PyAny> {
+        if self.is_instance(class).unwrap_or(false) {
+            Some(self)
+        } else {
+            None
+        }
     }
 
     fn is_python(&self) -> bool {
@@ -154,8 +153,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(tuple) = self.downcast::<PyTuple>() {
             Ok(PyArgs::new(Some(tuple), None).into())
         } else if let Ok(list) = self.downcast::<PyList>() {
-            let tuple = list_as_tuple(list);
-            Ok(PyArgs::new(Some(tuple), None).into())
+            Ok(PyArgs::new(Some(list.to_tuple()), None).into())
         } else {
             Err(ValError::new(ErrorType::ArgumentsType, self))
         }
@@ -188,26 +186,30 @@ impl<'a> Input<'a> for PyAny {
     }
 
     fn strict_str(&'a self) -> ValResult<EitherString<'a>> {
-        if let Ok(py_str) = self.downcast::<PyString>() {
-            if is_builtin_str(py_str) {
-                Ok(py_str.into())
-            } else {
-                Err(ValError::new(ErrorType::StringSubType, self))
-            }
+        if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
+            Ok(py_str.into())
+        } else if PyString::is_type_of(self) {
+            Err(ValError::new(ErrorType::StringSubType, self))
         } else {
             Err(ValError::new(ErrorType::StringType, self))
         }
     }
 
+    fn exact_int(&'a self) -> ValResult<EitherInt<'a>> {
+        if PyInt::is_exact_type_of(self) {
+            Ok(EitherInt::Py(self))
+        } else {
+            Err(ValError::new(ErrorType::IntType, self))
+        }
+    }
+
     fn lax_str(&'a self) -> ValResult<EitherString<'a>> {
-        if let Ok(py_str) = self.downcast::<PyString>() {
-            if is_builtin_str(py_str) {
-                Ok(py_str.into())
-            } else {
-                // force to a rust string to make sure behaviour is consistent whether or not we go via a
-                // rust string in StrConstrainedValidator - e.g. to_lower
-                Ok(py_string_str(py_str)?.into())
-            }
+        if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
+            Ok(py_str.into())
+        } else if let Ok(py_str) = self.downcast::<PyString>() {
+            // force to a rust string to make sure behaviour is consistent whether or not we go via a
+            // rust string in StrConstrainedValidator - e.g. to_lower
+            Ok(py_string_str(py_str)?.into())
         } else if let Ok(bytes) = self.downcast::<PyBytes>() {
             let str = match from_utf8(bytes.as_bytes()) {
                 Ok(s) => s,
@@ -227,14 +229,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn as_str_strict(&self) -> Option<&str> {
-        if self.get_type().is(get_py_str_type(self.py())) {
-            self.extract().ok()
-        } else {
-            None
-        }
-    }
-
     fn strict_bytes(&'a self) -> ValResult<EitherBytes<'a>> {
         if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             Ok(py_bytes.into())
@@ -257,23 +251,23 @@ impl<'a> Input<'a> for PyAny {
     }
 
     fn strict_bool(&self) -> ValResult<bool> {
-        if let Ok(bool) = self.extract::<bool>() {
-            Ok(bool)
+        if let Ok(bool) = self.downcast::<PyBool>() {
+            Ok(bool.is_true())
         } else {
             Err(ValError::new(ErrorType::BoolType, self))
         }
     }
 
     fn lax_bool(&self) -> ValResult<bool> {
-        if let Ok(bool) = self.extract::<bool>() {
-            Ok(bool)
+        if let Ok(bool) = self.downcast::<PyBool>() {
+            Ok(bool.is_true())
         } else if let Some(cow_str) = maybe_as_string(self, ErrorType::BoolParsing)? {
             str_as_bool(self, &cow_str)
-        } else if let Ok(int) = self.extract::<i64>() {
+        } else if let Ok(int) = extract_i64(self) {
             int_as_bool(self, int)
         } else if let Ok(float) = self.extract::<f64>() {
             match float_as_int(self, float) {
-                Ok(int) => int_as_bool(self, int),
+                Ok(int) => int.as_bool().ok_or_else(|| ValError::new(ErrorType::BoolParsing, self)),
                 _ => Err(ValError::new(ErrorType::BoolType, self)),
             }
         } else {
@@ -281,20 +275,24 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn strict_int(&self) -> ValResult<i64> {
-        // bool check has to come before int check as bools would be cast to ints below
-        if self.extract::<bool>().is_ok() {
-            Err(ValError::new(ErrorType::IntType, self))
-        } else if let Ok(int) = self.extract::<i64>() {
-            Ok(int)
+    fn strict_int(&'a self) -> ValResult<EitherInt<'a>> {
+        if PyInt::is_exact_type_of(self) {
+            Ok(EitherInt::Py(self))
+        } else if PyInt::is_type_of(self) {
+            // bools are a subclass of int, so check for bool type in this specific case
+            if PyBool::is_exact_type_of(self) {
+                Err(ValError::new(ErrorType::IntType, self))
+            } else {
+                Ok(EitherInt::Py(self))
+            }
         } else {
             Err(ValError::new(ErrorType::IntType, self))
         }
     }
 
-    fn lax_int(&self) -> ValResult<i64> {
-        if let Ok(int) = self.extract::<i64>() {
-            Ok(int)
+    fn lax_int(&'a self) -> ValResult<EitherInt<'a>> {
+        if PyInt::is_exact_type_of(self) {
+            Ok(EitherInt::Py(self))
         } else if let Some(cow_str) = maybe_as_string(self, ErrorType::IntParsing)? {
             str_as_int(self, &cow_str)
         } else if let Ok(float) = self.extract::<f64>() {
@@ -304,16 +302,8 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn as_int_strict(&self) -> Option<i64> {
-        if self.get_type().is(get_py_int_type(self.py())) {
-            self.extract().ok()
-        } else {
-            None
-        }
-    }
-
     fn ultra_strict_float(&self) -> ValResult<f64> {
-        if matches!(self.is_instance_of::<PyInt>(), Ok(true)) {
+        if self.is_instance_of::<PyInt>() {
             Err(ValError::new(ErrorType::FloatType, self))
         } else if let Ok(float) = self.extract::<f64>() {
             Ok(float)
@@ -322,10 +312,13 @@ impl<'a> Input<'a> for PyAny {
         }
     }
     fn strict_float(&self) -> ValResult<f64> {
-        if self.extract::<bool>().is_ok() {
-            Err(ValError::new(ErrorType::FloatType, self))
-        } else if let Ok(float) = self.extract::<f64>() {
-            Ok(float)
+        if let Ok(float) = self.extract::<f64>() {
+            // bools are cast to floats as either 0.0 or 1.0, so check for bool type in this specific case
+            if (float == 0.0 || float == 1.0) && PyBool::is_exact_type_of(self) {
+                Err(ValError::new(ErrorType::FloatType, self))
+            } else {
+                Ok(float)
+            }
         } else {
             Err(ValError::new(ErrorType::FloatType, self))
         }
@@ -515,7 +508,7 @@ impl<'a> Input<'a> for PyAny {
     }
 
     fn strict_date(&self) -> ValResult<EitherDate> {
-        if self.downcast::<PyDateTime>().is_ok() {
+        if PyDateTime::is_type_of(self) {
             // have to check if it's a datetime first, otherwise the line below converts to a date
             Err(ValError::new(ErrorType::DateType, self))
         } else if let Ok(date) = self.downcast::<PyDate>() {
@@ -526,7 +519,7 @@ impl<'a> Input<'a> for PyAny {
     }
 
     fn lax_date(&self) -> ValResult<EitherDate> {
-        if self.downcast::<PyDateTime>().is_ok() {
+        if PyDateTime::is_type_of(self) {
             // have to check if it's a datetime first, otherwise the line below converts to a date
             // even if we later try coercion from a datetime, we don't want to return a datetime now
             Err(ValError::new(ErrorType::DateType, self))
@@ -558,9 +551,9 @@ impl<'a> Input<'a> for PyAny {
             bytes_as_time(self, str.as_bytes())
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             bytes_as_time(self, py_bytes.as_bytes())
-        } else if self.downcast::<PyBool>().is_ok() {
+        } else if PyBool::is_exact_type_of(self) {
             Err(ValError::new(ErrorType::TimeType, self))
-        } else if let Ok(int) = self.extract::<i64>() {
+        } else if let Ok(int) = extract_i64(self) {
             int_as_time(self, int, 0)
         } else if let Ok(float) = self.extract::<f64>() {
             float_as_time(self, float)
@@ -585,9 +578,9 @@ impl<'a> Input<'a> for PyAny {
             bytes_as_datetime(self, str.as_bytes())
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             bytes_as_datetime(self, py_bytes.as_bytes())
-        } else if self.downcast::<PyBool>().is_ok() {
+        } else if PyBool::is_exact_type_of(self) {
             Err(ValError::new(ErrorType::DatetimeType, self))
-        } else if let Ok(int) = self.extract::<i64>() {
+        } else if let Ok(int) = extract_i64(self) {
             int_as_datetime(self, int, 0)
         } else if let Ok(float) = self.extract::<f64>() {
             float_as_datetime(self, float)
@@ -614,7 +607,7 @@ impl<'a> Input<'a> for PyAny {
             bytes_as_timedelta(self, str.as_bytes())
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             bytes_as_timedelta(self, py_bytes.as_bytes())
-        } else if let Ok(int) = self.extract::<i64>() {
+        } else if let Ok(int) = extract_i64(self) {
             Ok(int_as_duration(self, int)?.into())
         } else if let Ok(float) = self.extract::<f64>() {
             Ok(float_as_duration(self, float)?.into())
@@ -656,12 +649,8 @@ fn maybe_as_string(v: &PyAny, unicode_error: ErrorType) -> ValResult<Option<Cow<
     }
 }
 
-fn is_builtin_str(py_str: &PyString) -> bool {
-    py_str.get_type().is(PyString::type_object(py_str.py()))
-}
-
 #[cfg(PyPy)]
-static DICT_KEYS_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+static DICT_KEYS_TYPE: pyo3::once_cell::GILOnceCell<Py<PyType>> = pyo3::once_cell::GILOnceCell::new();
 
 #[cfg(PyPy)]
 fn is_dict_keys_type(v: &PyAny) -> bool {
@@ -679,7 +668,7 @@ fn is_dict_keys_type(v: &PyAny) -> bool {
 }
 
 #[cfg(PyPy)]
-static DICT_VALUES_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+static DICT_VALUES_TYPE: pyo3::once_cell::GILOnceCell<Py<PyType>> = pyo3::once_cell::GILOnceCell::new();
 
 #[cfg(PyPy)]
 fn is_dict_values_type(v: &PyAny) -> bool {
@@ -697,7 +686,7 @@ fn is_dict_values_type(v: &PyAny) -> bool {
 }
 
 #[cfg(PyPy)]
-static DICT_ITEMS_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+static DICT_ITEMS_TYPE: pyo3::once_cell::GILOnceCell<Py<PyType>> = pyo3::once_cell::GILOnceCell::new();
 
 #[cfg(PyPy)]
 fn is_dict_items_type(v: &PyAny) -> bool {
@@ -713,24 +702,3 @@ fn is_dict_items_type(v: &PyAny) -> bool {
         .as_ref(py);
     v.is_instance(items_type).unwrap_or(false)
 }
-
-pub fn list_as_tuple(list: &PyList) -> &PyTuple {
-    let py_tuple: Py<PyTuple> = unsafe {
-        let ptr = list.as_ptr();
-        let tuple_ptr = ffi::PyList_AsTuple(ptr);
-        Py::from_owned_ptr(list.py(), tuple_ptr)
-    };
-    py_tuple.into_ref(list.py())
-}
-
-static PY_INT_TYPE: GILOnceCell<PyObject> = GILOnceCell::new();
-
-fn get_py_int_type(py: Python) -> &PyObject {
-    PY_INT_TYPE.get_or_init(py, || PyInt::type_object(py).into())
-}
-
-static PY_STR_TYPE: GILOnceCell<PyObject> = GILOnceCell::new();
-
-fn get_py_str_type(py: Python) -> &PyObject {
-    PY_STR_TYPE.get_or_init(py, || PyString::type_object(py).into())
-}
