@@ -5,7 +5,7 @@ extern crate core;
 
 use pyo3::prelude::*;
 
-#[cfg(feature = "mimalloc")]
+#[cfg(feature = "mimalloc-allocator")]
 #[global_allocator]
 static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
 
@@ -18,18 +18,19 @@ mod lazy_index_map;
 mod lookup_key;
 mod recursion_guard;
 mod serializers;
+mod tools;
 mod url;
 mod validators;
 
 // required for benchmarks
 pub use self::url::{PyMultiHostUrl, PyUrl};
-pub use argument_markers::ArgsKwargs;
+pub use argument_markers::{ArgsKwargs, UndefinedType};
 pub use build_tools::SchemaError;
 pub use errors::{list_all_errors, PydanticCustomError, PydanticKnownError, PydanticOmit, ValidationError};
 pub use serializers::{
     to_json, to_jsonable_python, PydanticSerializationError, PydanticSerializationUnexpectedValue, SchemaSerializer,
 };
-pub use validators::SchemaValidator;
+pub use validators::{PySome, SchemaValidator};
 
 pub fn get_version() -> String {
     let version = env!("CARGO_PKG_VERSION");
@@ -42,9 +43,11 @@ pub fn get_version() -> String {
 }
 
 #[pymodule]
-fn _pydantic_core(_py: Python, m: &PyModule) -> PyResult<()> {
+fn _pydantic_core(py: Python, m: &PyModule) -> PyResult<()> {
     m.add("__version__", get_version())?;
     m.add("build_profile", env!("PROFILE"))?;
+    m.add("PydanticUndefined", UndefinedType::new(py))?;
+    m.add_class::<PySome>()?;
     m.add_class::<SchemaValidator>()?;
     m.add_class::<ValidationError>()?;
     m.add_class::<SchemaError>()?;
