@@ -1,4 +1,5 @@
 use ahash::AHashSet;
+use pyo3::exceptions::PyValueError;
 use std::hash::Hash;
 
 use pyo3::exceptions::PyTypeError;
@@ -6,7 +7,7 @@ use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyBool, PyDict, PySet};
 
-use crate::build_tools::SchemaDict;
+use crate::tools::SchemaDict;
 
 #[derive(Debug, Clone, Default)]
 pub(crate) struct SchemaFilter<T> {
@@ -14,6 +15,46 @@ pub(crate) struct SchemaFilter<T> {
     exclude: Option<AHashSet<T>>,
 }
 
+fn map_negative_index(value: &PyAny, len: Option<usize>) -> PyResult<&PyAny> {
+    let py = value.py();
+    match len {
+        Some(len) => Ok(value
+            .call_method1(intern!(py, "__mod__"), (len,))
+            .map_or_else(|_| value, |v| v)),
+        None => {
+            // check that it's not negative
+            let negative = value.call_method1(intern!(py, "__lt__"), (0,))?.is_true()?;
+            if negative {
+                Err(PyValueError::new_err(
+                    "Negative indices cannot be used to exclude items on unsized iterables",
+                ))
+            } else {
+                Ok(value)
+            }
+        }
+    }
+}
+
+fn map_negative_indices(include_or_exclude: &PyAny, len: Option<usize>) -> PyResult<&PyAny> {
+    let py = include_or_exclude.py();
+    if let Ok(exclude_dict) = include_or_exclude.downcast::<PyDict>() {
+        let out = PyDict::new(py);
+        for (k, v) in exclude_dict.iter() {
+            out.set_item(map_negative_index(k, len)?, v)?;
+        }
+        Ok(out)
+    } else if let Ok(exclude_set) = include_or_exclude.downcast::<PySet>() {
+        let mut values = Vec::with_capacity(exclude_set.len());
+        for v in exclude_set {
+            values.push(map_negative_index(v, len)?)
+        }
+        Ok(PySet::new(py, values)?)
+    } else {
+        // return as is and deal with the error later
+        Ok(include_or_exclude)
+    }
+}
+
 impl SchemaFilter<usize> {
     pub fn from_schema(schema: &PyDict) -> PyResult<Self> {
         let py = schema.py();
@@ -51,7 +92,10 @@ impl SchemaFilter<usize> {
         index: usize,
         include: Option<&'py PyAny>,
         exclude: Option<&'py PyAny>,
+        len: Option<usize>,
     ) -> PyResult<Option<(Option<&'py PyAny>, Option<&'py PyAny>)>> {
+        let include = include.map(|v| map_negative_indices(v, len)).transpose()?;
+        let exclude = exclude.map(|v| map_negative_indices(v, len)).transpose()?;
         self.filter(index, index, include, exclude)
     }
 }
@@ -232,7 +276,10 @@ impl AnyFilter {
         index: usize,
         include: Option<&'py PyAny>,
         exclude: Option<&'py PyAny>,
+        len: Option<usize>,
     ) -> PyResult<Option<(Option<&'py PyAny>, Option<&'py PyAny>)>> {
+        let include = include.map(|v| map_negative_indices(v, len)).transpose()?;
+        let exclude = exclude.map(|v| map_negative_indices(v, len)).transpose()?;
         self.filter(index, index, include, exclude)
     }
 }
