@@ -5,7 +5,7 @@ use pyo3::types::PyDict;
 
 use serde::ser::Serializer;
 
-use crate::build_context::BuildContext;
+use crate::definitions::DefinitionsBuilder;
 
 use super::{
     infer_json_key, infer_serialize, infer_to_python, BuildSerializer, CombinedSerializer, Extra, TypeSerializer,
@@ -20,7 +20,7 @@ impl BuildSerializer for AnySerializer {
     fn build(
         _schema: &PyDict,
         _config: Option<&PyDict>,
-        _build_context: &mut BuildContext<CombinedSerializer>,
+        _definitions: &mut DefinitionsBuilder<CombinedSerializer>,
     ) -> PyResult<CombinedSerializer> {
         Ok(Self {}.into())
     }
