@@ -1,18 +1,19 @@
 use std::borrow::Cow;
 
 use pyo3::prelude::*;
-use pyo3::types::{PyDict, PyString, PyType};
+use pyo3::types::{PyDict, PySet, PyString, PyType};
 use pyo3::{intern, PyTraverseError, PyVisit};
 
 use ahash::AHashMap;
 
-use crate::build_tools::{py_error_type, ExtraBehavior, SchemaDict};
+use crate::build_tools::{py_schema_error_type, ExtraBehavior};
 use crate::definitions::DefinitionsBuilder;
+use crate::tools::SchemaDict;
 
 use super::{
-    infer_json_key, infer_json_key_known, infer_serialize, infer_to_python, object_to_dict, py_err_se_err,
-    BuildSerializer, CombinedSerializer, ComputedFields, Extra, FieldsMode, GeneralFieldsSerializer, ObType, SerCheck,
-    SerField, TypeSerializer,
+    infer_json_key, infer_json_key_known, infer_serialize, infer_to_python, py_err_se_err, BuildSerializer,
+    CombinedSerializer, ComputedFields, Extra, FieldsMode, GeneralFieldsSerializer, ObType, SerCheck, SerField,
+    TypeSerializer,
 };
 
 const ROOT_FIELD: &str = "root";
@@ -51,7 +52,7 @@ impl BuildSerializer for ModelFieldsBuilder {
 
                 let schema = field_info.get_as_req(intern!(py, "schema"))?;
                 let serializer = CombinedSerializer::build(schema, config, definitions)
-                    .map_err(|e| py_error_type!("Field `{}`:\n  {}", key, e))?;
+                    .map_err(|e| py_schema_error_type!("Field `{}`:\n  {}", key, e))?;
 
                 fields.insert(key, SerField::new(py, key_py, alias, Some(serializer), true));
             }
@@ -109,23 +110,36 @@ fn has_extra(schema: &PyDict, config: Option<&PyDict>) -> PyResult<bool> {
 
 impl ModelSerializer {
     fn allow_value(&self, value: &PyAny, extra: &Extra) -> PyResult<bool> {
+        let class = self.class.as_ref(value.py());
         match extra.check {
-            SerCheck::Strict => Ok(value.get_type().is(self.class.as_ref(value.py()))),
-            SerCheck::Lax => value.is_instance(self.class.as_ref(value.py())),
+            SerCheck::Strict => Ok(value.get_type().is(class)),
+            SerCheck::Lax => value.is_instance(class),
             SerCheck::None => value.hasattr(intern!(value.py(), "__dict__")),
         }
     }
 
-    fn get_inner_value<'py>(&self, value: &'py PyAny, extra: &Extra) -> PyResult<&'py PyAny> {
-        let py = value.py();
-        let dict = object_to_dict(value, true, extra)?;
+    fn get_inner_value<'py>(&self, model: &'py PyAny, extra: &Extra) -> PyResult<&'py PyAny> {
+        let py = model.py();
+        let mut attrs: &PyDict = model.getattr(intern!(py, "__dict__"))?.downcast()?;
+
+        if extra.exclude_unset {
+            let fields_set: &PySet = model.getattr(intern!(py, "__pydantic_fields_set__"))?.downcast()?;
+
+            let new_attrs = attrs.copy()?;
+            for key in new_attrs.keys() {
+                if !fields_set.contains(key)? {
+                    new_attrs.del_item(key)?;
+                }
+            }
+            attrs = new_attrs;
+        }
 
         if self.has_extra {
-            let model_extra = value.getattr(intern!(py, "__pydantic_extra__"))?;
-            let py_tuple = (dict, model_extra).to_object(py);
+            let model_extra = model.getattr(intern!(py, "__pydantic_extra__"))?;
+            let py_tuple = (attrs, model_extra).to_object(py);
             Ok(py_tuple.into_ref(py))
         } else {
-            Ok(dict)
+            Ok(attrs)
         }
     }
 }
