@@ -5,8 +5,8 @@ use std::borrow::Cow;
 
 use serde::ser::SerializeSeq;
 
-use crate::build_tools::SchemaDict;
 use crate::definitions::DefinitionsBuilder;
+use crate::tools::SchemaDict;
 
 use super::any::AnySerializer;
 use super::{
@@ -62,7 +62,9 @@ impl TypeSerializer for TupleVariableSerializer {
 
                 let mut items = Vec::with_capacity(py_tuple.len());
                 for (index, element) in py_tuple.iter().enumerate() {
-                    let op_next = self.filter.index_filter(index, include, exclude)?;
+                    let op_next = self
+                        .filter
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))?;
                     if let Some((next_include, next_exclude)) = op_next {
                         items.push(item_serializer.to_python(element, next_include, next_exclude, extra)?);
                     }
@@ -114,7 +116,7 @@ impl TypeSerializer for TupleVariableSerializer {
                 for (index, element) in py_tuple.iter().enumerate() {
                     let op_next = self
                         .filter
-                        .index_filter(index, include, exclude)
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))
                         .map_err(py_err_se_err)?;
                     if let Some((next_include, next_exclude)) = op_next {
                         let item_serialize =
@@ -198,7 +200,9 @@ impl TypeSerializer for TuplePositionalSerializer {
                         Some(value) => value,
                         None => break,
                     };
-                    let op_next = self.filter.index_filter(index, include, exclude)?;
+                    let op_next = self
+                        .filter
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))?;
                     if let Some((next_include, next_exclude)) = op_next {
                         items.push(serializer.to_python(element, next_include, next_exclude, extra)?);
                     }
@@ -207,7 +211,9 @@ impl TypeSerializer for TuplePositionalSerializer {
                 let extra_serializer = self.extra_serializer.as_ref();
                 for (index2, element) in py_tuple_iter.enumerate() {
                     let index = index2 + expected_length;
-                    let op_next = self.filter.index_filter(index, include, exclude)?;
+                    let op_next = self
+                        .filter
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))?;
                     if let Some((next_include, next_exclude)) = op_next {
                         items.push(extra_serializer.to_python(element, next_include, next_exclude, extra)?);
                     }
@@ -272,7 +278,7 @@ impl TypeSerializer for TuplePositionalSerializer {
                     };
                     let op_next = self
                         .filter
-                        .index_filter(index, include, exclude)
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))
                         .map_err(py_err_se_err)?;
                     if let Some((next_include, next_exclude)) = op_next {
                         let item_serialize =
@@ -287,7 +293,7 @@ impl TypeSerializer for TuplePositionalSerializer {
                     let index = index2 + expected_length;
                     let op_next = self
                         .filter
-                        .index_filter(index, include, exclude)
+                        .index_filter(index, include, exclude, Some(py_tuple.len()))
                         .map_err(py_err_se_err)?;
                     if let Some((next_include, next_exclude)) = op_next {
                         let item_serialize =
