@@ -12,7 +12,7 @@ use crate::SchemaValidator;
 
 static SCHEMA_DEFINITION_URL: GILOnceCell<SchemaValidator> = GILOnceCell::new();
 
-#[pyclass(name = "Url", module = "pydantic_core._pydantic_core")]
+#[pyclass(name = "Url", module = "pydantic_core._pydantic_core", subclass)]
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct PyUrl {
@@ -41,7 +41,7 @@ impl PyUrl {
     pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
         let schema_obj = SCHEMA_DEFINITION_URL
             .get_or_init(py, || build_schema_validator(py, "url"))
-            .validate_python(py, url, None, None, None)?;
+            .validate_python(py, url, None, None, None, None)?;
         schema_obj.extract(py)
     }
 
@@ -147,7 +147,7 @@ impl PyUrl {
     }
 }
 
-#[pyclass(name = "MultiHostUrl", module = "pydantic_core._pydantic_core")]
+#[pyclass(name = "MultiHostUrl", module = "pydantic_core._pydantic_core", subclass)]
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct PyMultiHostUrl {
@@ -176,7 +176,7 @@ impl PyMultiHostUrl {
     pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
         let schema_obj = SCHEMA_DEFINITION_MULTI_HOST_URL
             .get_or_init(py, || build_schema_validator(py, "multi-host-url"))
-            .validate_python(py, url, None, None, None)?;
+            .validate_python(py, url, None, None, None, None)?;
         schema_obj.extract(py)
     }
 
