@@ -4,11 +4,13 @@ use pyo3::types::{PyDict, PyList, PyString, PyTuple};
 
 use ahash::AHashSet;
 
-use crate::build_tools::{py_err, schema_or_config_same, SchemaDict};
+use crate::build_tools::py_schema_err;
+use crate::build_tools::schema_or_config_same;
 use crate::errors::{ErrorType, ValError, ValLineError, ValResult};
 use crate::input::{GenericArguments, Input};
 use crate::lookup_key::LookupKey;
 use crate::recursion_guard::RecursionGuard;
+use crate::tools::SchemaDict;
 
 use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
 
@@ -77,13 +79,13 @@ impl BuildValidator for ArgumentsValidator {
 
             let validator = match build_validator(schema, config, definitions) {
                 Ok(v) => v,
-                Err(err) => return py_err!("Parameter '{}':\n  {}", name, err),
+                Err(err) => return py_schema_err!("Parameter '{}':\n  {}", name, err),
             };
 
             let has_default = match validator {
                 CombinedValidator::WithDefault(ref v) => {
                     if v.omit_on_error() {
-                        return py_err!("Parameter '{}': omit_on_error cannot be used with arguments", name);
+                        return py_schema_err!("Parameter '{}': omit_on_error cannot be used with arguments", name);
                     }
                     v.has_default()
                 }
@@ -91,7 +93,7 @@ impl BuildValidator for ArgumentsValidator {
             };
 
             if had_default_arg && !has_default {
-                return py_err!("Non-default argument '{}' follows default argument", name);
+                return py_schema_err!("Non-default argument '{}' follows default argument", name);
             } else if has_default {
                 had_default_arg = true;
             }
