@@ -5,12 +5,13 @@ use pyo3::types::{PyDict, PyList, PyString, PyTuple, PyType};
 
 use ahash::AHashSet;
 
-use crate::build_tools::{is_strict, py_err, schema_or_config_same, ExtraBehavior, SchemaDict};
+use crate::build_tools::py_schema_err;
+use crate::build_tools::{is_strict, schema_or_config_same, ExtraBehavior};
 use crate::errors::{ErrorType, ValError, ValLineError, ValResult};
-use crate::input::InputType;
 use crate::input::{GenericArguments, Input};
 use crate::lookup_key::LookupKey;
 use crate::recursion_guard::RecursionGuard;
+use crate::tools::SchemaDict;
 use crate::validators::function::convert_err;
 
 use super::arguments::{json_get, json_slice, py_get, py_slice};
@@ -76,12 +77,12 @@ impl BuildValidator for DataclassArgsValidator {
 
             let validator = match build_validator(schema, config, definitions) {
                 Ok(v) => v,
-                Err(err) => return py_err!("Field '{}':\n  {}", name, err),
+                Err(err) => return py_schema_err!("Field '{}':\n  {}", name, err),
             };
 
             if let CombinedValidator::WithDefault(ref v) = validator {
                 if v.omit_on_error() {
-                    return py_err!("Field `{}`: omit_on_error cannot be used with arguments", name);
+                    return py_schema_err!("Field `{}`: omit_on_error cannot be used with arguments", name);
                 }
             }
 
@@ -411,10 +412,12 @@ pub struct DataclassValidator {
     strict: bool,
     validator: Box<CombinedValidator>,
     class: Py<PyType>,
+    fields: Vec<Py<PyString>>,
     post_init: Option<Py<PyString>>,
     revalidate: Revalidate,
     name: String,
     frozen: bool,
+    slots: bool,
 }
 
 impl BuildValidator for DataclassValidator {
@@ -422,11 +425,14 @@ impl BuildValidator for DataclassValidator {
 
     fn build(
         schema: &PyDict,
-        config: Option<&PyDict>,
+        _config: Option<&PyDict>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
 
+        // dataclasses ignore the parent config and always use the config from this dataclasses
+        let config = schema.get_as(intern!(py, "config"))?;
+
         let class: &PyType = schema.get_as_req(intern!(py, "cls"))?;
         let name = match schema.get_as_req::<String>(intern!(py, "cls_name")) {
             Ok(name) => name,
@@ -441,10 +447,17 @@ impl BuildValidator for DataclassValidator {
             None
         };
 
+        let fields = schema
+            .get_as_req::<&PyList>(intern!(py, "fields"))?
+            .iter()
+            .map(|s| Ok(s.downcast::<PyString>()?.into_py(py)))
+            .collect::<PyResult<Vec<_>>>()?;
+
         Ok(Self {
             strict: is_strict(schema, config)?,
             validator: Box::new(validator),
             class: class.into(),
+            fields,
             post_init,
             revalidate: Revalidate::from_str(schema_or_config_same(
                 schema,
@@ -453,6 +466,7 @@ impl BuildValidator for DataclassValidator {
             )?)?,
             name,
             frozen: schema.get_as(intern!(py, "frozen"))?.unwrap_or(false),
+            slots: schema.get_as(intern!(py, "slots"))?.unwrap_or(false),
         }
         .into())
     }
@@ -474,12 +488,12 @@ impl Validator for DataclassValidator {
 
         // same logic as on models
         let class = self.class.as_ref(py);
-        if matches!(extra.mode, InputType::Python) && input.to_object(py).as_ref(py).is_instance(class)? {
-            if self.revalidate.should_revalidate(input, class) {
-                let input = input.input_get_attr(intern!(py, "__dict__")).unwrap()?;
+        if let Some(py_input) = input.input_is_instance(class) {
+            if self.revalidate.should_revalidate(py_input, class) {
+                let input_dict: &PyAny = self.dataclass_to_dict(py, py_input)?;
                 let val_output = self
                     .validator
-                    .validate(py, input, extra, definitions, recursion_guard)?;
+                    .validate(py, input_dict, extra, definitions, recursion_guard)?;
                 let dc = create_class(self.class.as_ref(py))?;
                 self.set_dict_call(py, dc.as_ref(py), val_output, input)?;
                 Ok(dc)
@@ -516,14 +530,11 @@ impl Validator for DataclassValidator {
         if self.frozen {
             return Err(ValError::new(ErrorType::FrozenInstance, field_value));
         }
-        let dict_py_str = intern!(py, "__dict__");
-        let dict: &PyDict = obj.getattr(dict_py_str)?.downcast()?;
 
-        let new_dict = dict.copy()?;
+        let new_dict = self.dataclass_to_dict(py, obj)?;
+
         new_dict.set_item(field_name, field_value)?;
 
-        // Discard the second return value, which is `init_only_args` but is always
-        // None anyway for validate_assignment; see validate_assignment in DataclassArgsValidator
         let val_assignment_result = self.validator.validate_assignment(
             py,
             new_dict,
@@ -536,7 +547,14 @@ impl Validator for DataclassValidator {
 
         let (dc_dict, _): (&PyDict, PyObject) = val_assignment_result.extract(py)?;
 
-        force_setattr(py, obj, dict_py_str, dc_dict)?;
+        if self.slots {
+            let value = dc_dict
+                .get_item(field_name)
+                .ok_or_else(|| PyKeyError::new_err(field_name.to_string()))?;
+            force_setattr(py, obj, field_name, value)?;
+        } else {
+            force_setattr(py, obj, intern!(py, "__dict__"), dc_dict)?;
+        }
 
         Ok(obj.to_object(py))
     }
@@ -587,6 +605,17 @@ impl DataclassValidator {
 
         Ok(self_instance.into_py(py))
     }
+
+    fn dataclass_to_dict<'py>(&self, py: Python<'py>, dc: &'py PyAny) -> PyResult<&'py PyDict> {
+        let dict = PyDict::new(py);
+
+        for field_name in &self.fields {
+            let field_name = field_name.as_ref(py);
+            dict.set_item(field_name, dc.getattr(field_name)?)?;
+        }
+        Ok(dict)
+    }
+
     fn set_dict_call<'s, 'data>(
         &'s self,
         py: Python<'data>,
@@ -595,7 +624,14 @@ impl DataclassValidator {
         input: &'data impl Input<'data>,
     ) -> ValResult<'data, ()> {
         let (dc_dict, post_init_kwargs): (&PyAny, &PyAny) = val_output.extract(py)?;
-        force_setattr(py, dc, intern!(py, "__dict__"), dc_dict)?;
+        if self.slots {
+            let dc_dict: &PyDict = dc_dict.downcast()?;
+            for (key, value) in dc_dict.iter() {
+                force_setattr(py, dc, key, value)?;
+            }
+        } else {
+            force_setattr(py, dc, intern!(py, "__dict__"), dc_dict)?;
+        }
 
         if let Some(ref post_init) = self.post_init {
             let post_init = post_init.as_ref(py);
