@@ -5,14 +5,15 @@ use enum_dispatch::enum_dispatch;
 use pyo3::exceptions::PyTypeError;
 use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
-use pyo3::types::{PyAny, PyDict};
+use pyo3::types::{PyAny, PyDict, PyTuple, PyType};
 use pyo3::{intern, PyTraverseError, PyVisit};
 
-use crate::build_tools::{py_err, py_error_type, SchemaDict, SchemaError};
+use crate::build_tools::{py_schema_err, py_schema_error_type, SchemaError};
 use crate::definitions::{Definitions, DefinitionsBuilder};
 use crate::errors::{ErrorMode, LocItem, ValError, ValResult, ValidationError};
 use crate::input::{Input, InputType};
 use crate::recursion_guard::RecursionGuard;
+use crate::tools::SchemaDict;
 
 mod any;
 mod arguments;
@@ -57,6 +58,40 @@ pub use with_default::DefaultType;
 
 use self::definitions::DefinitionRefValidator;
 
+#[pyclass(module = "pydantic_core._pydantic_core", name = "Some")]
+pub struct PySome {
+    #[pyo3(get)]
+    value: PyObject,
+}
+
+impl PySome {
+    fn new(value: PyObject) -> Self {
+        Self { value }
+    }
+}
+
+#[pymethods]
+impl PySome {
+    pub fn __repr__(&self, py: Python) -> PyResult<String> {
+        Ok(format!("Some({})", self.value.as_ref(py).repr()?,))
+    }
+
+    #[new]
+    pub fn py_new(value: PyObject) -> Self {
+        Self { value }
+    }
+
+    #[classmethod]
+    pub fn __class_getitem__(cls: &PyType, _args: &PyAny) -> Py<PyType> {
+        cls.into_py(cls.py())
+    }
+
+    #[classattr]
+    fn __match_args__(py: Python) -> &PyTuple {
+        PyTuple::new(py, vec![intern!(py, "value")])
+    }
+}
+
 #[pyclass(module = "pydantic_core._pydantic_core")]
 #[derive(Debug, Clone)]
 pub struct SchemaValidator {
@@ -65,6 +100,7 @@ pub struct SchemaValidator {
     schema: PyObject,
     #[pyo3(get)]
     title: PyObject,
+    hide_input_in_errors: bool,
 }
 
 #[pymethods]
@@ -90,11 +126,13 @@ impl SchemaValidator {
             Some(t) => t.into_py(py),
             None => validator.get_name().into_py(py),
         };
+        let hide_input_in_errors: bool = config.get_as(intern!(py, "hide_input_in_errors"))?.unwrap_or(false);
         Ok(Self {
             validator,
             definitions,
             schema: schema.into_py(py),
             title,
+            hide_input_in_errors,
         })
     }
 
@@ -104,29 +142,47 @@ impl SchemaValidator {
         Ok((cls, args).into_py(py))
     }
 
-    #[pyo3(signature = (input, *, strict=None, context=None, self_instance=None))]
+    #[pyo3(signature = (input, *, strict=None, from_attributes=None, context=None, self_instance=None))]
     pub fn validate_python(
         &self,
         py: Python,
         input: &PyAny,
         strict: Option<bool>,
+        from_attributes: Option<bool>,
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<PyObject> {
-        let r = self._validate(py, input, InputType::Python, strict, context, self_instance);
+        let r = self._validate(
+            py,
+            input,
+            InputType::Python,
+            strict,
+            from_attributes,
+            context,
+            self_instance,
+        );
         r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
-    #[pyo3(signature = (input, *, strict=None, context=None, self_instance=None))]
+    #[pyo3(signature = (input, *, strict=None, from_attributes=None, context=None, self_instance=None))]
     pub fn isinstance_python(
         &self,
         py: Python,
         input: &PyAny,
         strict: Option<bool>,
+        from_attributes: Option<bool>,
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<bool> {
-        match self._validate(py, input, InputType::Python, strict, context, self_instance) {
+        match self._validate(
+            py,
+            input,
+            InputType::Python,
+            strict,
+            from_attributes,
+            context,
+            self_instance,
+        ) {
             Ok(_) => Ok(true),
             Err(ValError::InternalErr(err)) => Err(err),
             Err(ValError::Omit) => Err(ValidationError::omit_error()),
@@ -145,14 +201,15 @@ impl SchemaValidator {
     ) -> PyResult<PyObject> {
         match input.parse_json() {
             Ok(input) => {
-                let r = self._validate(py, &input, InputType::Json, strict, context, self_instance);
+                let r = self._validate(py, &input, InputType::Json, strict, None, context, self_instance);
                 r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Json))
             }
             Err(err) => Err(self.prepare_validation_err(py, err, ErrorMode::Json)),
         }
     }
 
-    #[pyo3(signature = (obj, field_name, field_value, *, strict=None, context=None))]
+    #[allow(clippy::too_many_arguments)]
+    #[pyo3(signature = (obj, field_name, field_value, *, strict=None, from_attributes=None, context=None))]
     pub fn validate_assignment(
         &self,
         py: Python,
@@ -160,12 +217,14 @@ impl SchemaValidator {
         field_name: &str,
         field_value: &PyAny,
         strict: Option<bool>,
+        from_attributes: Option<bool>,
         context: Option<&PyAny>,
     ) -> PyResult<PyObject> {
         let extra = Extra {
             mode: InputType::Python,
             data: None,
             strict,
+            from_attributes,
             ultra_strict: false,
             context,
             field_name: None,
@@ -178,6 +237,31 @@ impl SchemaValidator {
             .map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
+    #[pyo3(signature = (*, strict=None, context=None))]
+    pub fn get_default_value(&self, py: Python, strict: Option<bool>, context: Option<&PyAny>) -> PyResult<PyObject> {
+        let extra = Extra {
+            mode: InputType::Python,
+            data: None,
+            strict,
+            from_attributes: None,
+            ultra_strict: false,
+            context,
+            field_name: None,
+            self_instance: None,
+        };
+        let recursion_guard = &mut RecursionGuard::default();
+        let r = self
+            .validator
+            .default_value(py, None::<i64>, &extra, &self.definitions, recursion_guard);
+        match r {
+            Ok(maybe_default) => match maybe_default {
+                Some(v) => Ok(PySome::new(v).into_py(py)),
+                None => Ok(py.None().into_py(py)),
+            },
+            Err(e) => Err(self.prepare_validation_err(py, e, ErrorMode::Python)),
+        }
+    }
+
     pub fn __repr__(&self, py: Python) -> String {
         format!(
             "SchemaValidator(title={:?}, validator={:#?}, definitions={:#?})",
@@ -205,12 +289,14 @@ impl SchemaValidator {
 }
 
 impl SchemaValidator {
+    #[allow(clippy::too_many_arguments)]
     fn _validate<'s, 'data>(
         &'data self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
         mode: InputType,
         strict: Option<bool>,
+        from_attributes: Option<bool>,
         context: Option<&'data PyAny>,
         self_instance: Option<&PyAny>,
     ) -> ValResult<'data, PyObject>
@@ -220,14 +306,21 @@ impl SchemaValidator {
         self.validator.validate(
             py,
             input,
-            &Extra::new(strict, context, self_instance, mode),
+            &Extra::new(strict, from_attributes, context, self_instance, mode),
             &self.definitions,
             &mut RecursionGuard::default(),
         )
     }
 
     fn prepare_validation_err(&self, py: Python, error: ValError, error_mode: ErrorMode) -> PyErr {
-        ValidationError::from_val_error(py, self.title.clone_ref(py), error_mode, error, None)
+        ValidationError::from_val_error(
+            py,
+            self.title.clone_ref(py),
+            error_mode,
+            error,
+            None,
+            self.hide_input_in_errors,
+        )
     }
 }
 
@@ -248,7 +341,7 @@ impl<'py> SelfValidator<'py> {
     }
 
     pub fn validate_schema(&self, py: Python<'py>, schema: &'py PyAny) -> PyResult<&'py PyAny> {
-        let extra = Extra::new(None, None, None, InputType::Python);
+        let extra = Extra::new(None, None, None, None, InputType::Python);
         match self.validator.validator.validate(
             py,
             schema,
@@ -271,7 +364,7 @@ impl<'py> SelfValidator<'py> {
 
         let mut validator = match build_validator(self_schema, None, &mut definitions_builder) {
             Ok(v) => v,
-            Err(err) => return py_err!("Error building self-schema:\n  {}", err),
+            Err(err) => return py_schema_err!("Error building self-schema:\n  {}", err),
         };
         validator.complete(&definitions_builder)?;
         let mut definitions = definitions_builder.clone().finish()?;
@@ -283,6 +376,7 @@ impl<'py> SelfValidator<'py> {
             definitions,
             schema: py.None(),
             title: "Self Schema".into_py(py),
+            hide_input_in_errors: false,
         })
     }
 }
@@ -314,7 +408,7 @@ fn build_specific_validator<'a, T: BuildValidator>(
     }
 
     T::build(schema_dict, config, definitions)
-        .map_err(|err| py_error_type!("Error building \"{}\" validator:\n  {}", val_type, err))
+        .map_err(|err| py_schema_error_type!("Error building \"{}\" validator:\n  {}", val_type, err))
 }
 
 // macro to build the match statement for validator selection
@@ -324,7 +418,7 @@ macro_rules! validator_match {
             $(
                 <$validator>::EXPECTED_TYPE => build_specific_validator::<$validator>($type, $dict, $config, $definitions),
             )+
-            _ => return py_err!(r#"Unknown schema type: "{}""#, $type),
+            _ => return py_schema_err!(r#"Unknown schema type: "{}""#, $type),
         }
     };
 }
@@ -440,6 +534,8 @@ pub struct Extra<'a> {
     pub strict: Option<bool>,
     /// whether we're in ultra-strict mode, only used occasionally in unions
     pub ultra_strict: bool,
+    /// Validation time setting of `from_attributes`
+    pub from_attributes: Option<bool>,
     /// context used in validator functions
     pub context: Option<&'a PyAny>,
     /// This is an instance of the model or dataclass being validated, when validation is performed from `__init__`
@@ -449,18 +545,20 @@ pub struct Extra<'a> {
 impl<'a> Extra<'a> {
     pub fn new(
         strict: Option<bool>,
+        from_attributes: Option<bool>,
         context: Option<&'a PyAny>,
         self_instance: Option<&'a PyAny>,
         mode: InputType,
     ) -> Self {
         Extra {
-            strict,
-            context,
-            self_instance,
             mode,
             data: None,
             field_name: None,
+            strict,
             ultra_strict: false,
+            from_attributes,
+            context,
+            self_instance,
         }
     }
 }
@@ -472,6 +570,7 @@ impl<'a> Extra<'a> {
             data: self.data,
             strict: Some(true),
             ultra_strict,
+            from_attributes: self.from_attributes,
             context: self.context,
             field_name: self.field_name,
             self_instance: self.self_instance,
