@@ -6,18 +6,21 @@ use pyo3::prelude::*;
 use pyo3::types::{PyDict, PySet, PyString, PyTuple, PyType};
 use pyo3::{ffi, intern};
 
-use crate::build_tools::{py_err, schema_or_config_same, SchemaDict};
-use crate::errors::{ErrorType, ValError, ValResult};
-use crate::input::{py_error_on_minusone, Input, InputType};
-use crate::recursion_guard::RecursionGuard;
-
 use super::function::convert_err;
 use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use crate::build_tools::py_schema_err;
+use crate::build_tools::schema_or_config_same;
+use crate::errors::{ErrorType, ValError, ValResult};
+use crate::input::{py_error_on_minusone, Input};
+use crate::recursion_guard::RecursionGuard;
+use crate::tools::{py_err, SchemaDict};
+use crate::UndefinedType;
 
 const ROOT_FIELD: &str = "root";
 const DUNDER_DICT: &str = "__dict__";
 const DUNDER_FIELDS_SET_KEY: &str = "__pydantic_fields_set__";
 const DUNDER_MODEL_EXTRA_KEY: &str = "__pydantic_extra__";
+const DUNDER_MODEL_PRIVATE_KEY: &str = "__pydantic_private__";
 
 #[derive(Debug, Clone)]
 pub(super) enum Revalidate {
@@ -33,22 +36,21 @@ impl Revalidate {
             Some("always") => Ok(Self::Always),
             Some("never") => Ok(Self::Never),
             Some("subclass-instances") => Ok(Self::SubclassInstances),
-            Some(s) => py_err!("Invalid revalidate_instances value: {}", s),
+            Some(s) => py_schema_err!("Invalid revalidate_instances value: {}", s),
         }
     }
 
-    pub fn should_revalidate<'d>(&self, input: &impl Input<'d>, class: &PyType) -> bool {
+    pub fn should_revalidate(&self, input: &PyAny, class: &PyType) -> bool {
         match self {
             Revalidate::Always => true,
             Revalidate::Never => false,
-            Revalidate::SubclassInstances => !input.is_exact_instance(class),
+            Revalidate::SubclassInstances => !input.get_type().is(class),
         }
     }
 }
 
 #[derive(Debug, Clone)]
 pub struct ModelValidator {
-    strict: bool,
     revalidate: Revalidate,
     validator: Box<CombinedValidator>,
     class: Py<PyType>,
@@ -76,9 +78,6 @@ impl BuildValidator for ModelValidator {
         let validator = build_validator(sub_schema, config, definitions)?;
 
         Ok(Self {
-            // we don't use is_strict here since we don't want validation to be strict in this case if
-            // `config.strict` is set, only if this specific field is strict
-            strict: schema.get_as(intern!(py, "strict"))?.unwrap_or(false),
             revalidate: Revalidate::from_str(schema_or_config_same(
                 schema,
                 config,
@@ -125,16 +124,16 @@ impl Validator for ModelValidator {
         // if the input is an instance of the class, we "revalidate" it - e.g. we extract and reuse `__pydantic_fields_set__`
         // but use from attributes to create a new instance of the model field type
         let class = self.class.as_ref(py);
-        if matches!(extra.mode, InputType::Python) && input.to_object(py).as_ref(py).is_instance(class)? {
-            if self.revalidate.should_revalidate(input, class) {
+        if let Some(py_input) = input.input_is_instance(class) {
+            if self.revalidate.should_revalidate(py_input, class) {
+                let fields_set = py_input.getattr(intern!(py, DUNDER_FIELDS_SET_KEY))?;
                 if self.root_model {
-                    let inner_input: &PyAny = input.input_get_attr(intern!(py, ROOT_FIELD)).unwrap()?;
-                    self.validate_construct(py, inner_input, None, extra, definitions, recursion_guard)
+                    let inner_input = py_input.getattr(intern!(py, ROOT_FIELD))?;
+                    self.validate_construct(py, inner_input, Some(fields_set), extra, definitions, recursion_guard)
                 } else {
-                    let fields_set = input.input_get_attr(intern!(py, DUNDER_FIELDS_SET_KEY)).unwrap()?;
                     // get dict here so from_attributes logic doesn't apply
-                    let dict = input.input_get_attr(intern!(py, DUNDER_DICT)).unwrap()?;
-                    let model_extra = input.input_get_attr(intern!(py, DUNDER_MODEL_EXTRA_KEY)).unwrap()?;
+                    let dict = py_input.getattr(intern!(py, DUNDER_DICT))?;
+                    let model_extra = py_input.getattr(intern!(py, DUNDER_MODEL_EXTRA_KEY))?;
 
                     let inner_input: &PyAny = if model_extra.is_none() {
                         dict
@@ -148,13 +147,6 @@ impl Validator for ModelValidator {
             } else {
                 Ok(input.to_object(py))
             }
-        } else if extra.strict.unwrap_or(self.strict) && input.is_python() {
-            Err(ValError::new(
-                ErrorType::ModelClassType {
-                    class_name: self.get_name().to_string(),
-                },
-                input,
-            ))
         } else {
             self.validate_construct(py, input, None, extra, definitions, recursion_guard)
         }
@@ -211,7 +203,7 @@ impl Validator for ModelValidator {
 
         let (output, _, updated_fields_set): (&PyDict, &PyAny, &PySet) = output.extract(py)?;
 
-        if let Ok(fields_set) = model.input_get_attr(intern!(py, DUNDER_FIELDS_SET_KEY)).unwrap() {
+        if let Ok(fields_set) = model.getattr(intern!(py, DUNDER_FIELDS_SET_KEY)) {
             let fields_set: &PySet = fields_set.downcast()?;
             for field_name in updated_fields_set {
                 fields_set.add(field_name)?;
@@ -267,6 +259,12 @@ impl ModelValidator {
             .validate(py, input, &new_extra, definitions, recursion_guard)?;
 
         if self.root_model {
+            let fields_set = if input.to_object(py).is(&UndefinedType::py_undefined()) {
+                PySet::empty(py)?
+            } else {
+                PySet::new(py, [&String::from(ROOT_FIELD)])?
+            };
+            force_setattr(py, self_instance, intern!(py, DUNDER_FIELDS_SET_KEY), fields_set)?;
             force_setattr(py, self_instance, intern!(py, ROOT_FIELD), output.as_ref(py))?;
         } else {
             let (model_dict, model_extra, fields_set): (&PyAny, &PyAny, &PyAny) = output.extract(py)?;
@@ -361,6 +359,7 @@ fn set_model_attrs(instance: &PyAny, model_dict: &PyAny, model_extra: &PyAny, fi
     let py = instance.py();
     force_setattr(py, instance, intern!(py, DUNDER_DICT), model_dict)?;
     force_setattr(py, instance, intern!(py, DUNDER_MODEL_EXTRA_KEY), model_extra)?;
+    force_setattr(py, instance, intern!(py, DUNDER_MODEL_PRIVATE_KEY), py.None())?;
     force_setattr(py, instance, intern!(py, DUNDER_FIELDS_SET_KEY), fields_set)?;
     Ok(())
 }
