@@ -1,21 +1,19 @@
-use std::borrow::Cow;
-use std::fmt;
 use std::fmt::Write;
 
-use pyo3::exceptions::PyTypeError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList, PyString};
 
-use ahash::AHashMap;
-
-use crate::build_tools::{is_strict, py_err, schema_or_config, SchemaDict};
+use crate::build_tools::py_schema_err;
+use crate::build_tools::{is_strict, schema_or_config};
 use crate::errors::{ErrorType, LocItem, ValError, ValLineError, ValResult};
 use crate::input::{GenericMapping, Input};
 use crate::lookup_key::LookupKey;
 use crate::recursion_guard::RecursionGuard;
+use crate::tools::SchemaDict;
 
 use super::custom_error::CustomError;
+use super::literal::LiteralLookup;
 use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
 
 #[derive(Debug, Clone)]
@@ -45,7 +43,7 @@ impl BuildValidator for UnionValidator {
 
         let auto_collapse = || schema.get_as_req(intern!(py, "auto_collapse")).unwrap_or(true);
         match choices.len() {
-            0 => py_err!("One or more union choices required"),
+            0 => py_schema_err!("One or more union choices required"),
             1 if auto_collapse() => Ok(choices.into_iter().next().unwrap()),
             _ => {
                 let descr = choices.iter().map(|v| v.get_name()).collect::<Vec<_>>().join(",");
@@ -218,54 +216,10 @@ impl Discriminator {
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq, Hash)]
-enum ChoiceKey {
-    Int(i64),
-    Str(String),
-}
-
-impl ChoiceKey {
-    fn from_py(raw: &PyAny) -> PyResult<Self> {
-        if let Ok(py_int) = raw.extract::<i64>() {
-            Ok(Self::Int(py_int))
-        } else if let Ok(py_str) = raw.downcast::<PyString>() {
-            Ok(Self::Str(py_str.to_str()?.to_string()))
-        } else {
-            py_err!(PyTypeError; "Expected int or str, got {}", raw.get_type().name().unwrap_or("<unknown python object>"))
-        }
-    }
-
-    fn repr(&self) -> String {
-        match self {
-            Self::Int(i) => i.to_string(),
-            Self::Str(s) => format!("'{s}'"),
-        }
-    }
-}
-
-impl fmt::Display for ChoiceKey {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            Self::Int(i) => write!(f, "{i}"),
-            Self::Str(s) => write!(f, "{s}"),
-        }
-    }
-}
-
-impl From<&ChoiceKey> for LocItem {
-    fn from(key: &ChoiceKey) -> Self {
-        match key {
-            ChoiceKey::Str(s) => s.as_str().into(),
-            ChoiceKey::Int(i) => (*i).into(),
-        }
-    }
-}
-
 #[derive(Debug, Clone)]
 pub struct TaggedUnionValidator {
-    choices: AHashMap<ChoiceKey, CombinedValidator>,
-    repeat_choices: Option<AHashMap<ChoiceKey, ChoiceKey>>,
     discriminator: Discriminator,
+    lookup: LiteralLookup<CombinedValidator>,
     from_attributes: bool,
     strict: bool,
     custom_error: Option<CustomError>,
@@ -286,23 +240,17 @@ impl BuildValidator for TaggedUnionValidator {
         let discriminator = Discriminator::new(py, schema.get_as_req(intern!(py, "discriminator"))?)?;
         let discriminator_repr = discriminator.to_string_py(py)?;
 
-        let schema_choices: &PyDict = schema.get_as_req(intern!(py, "choices"))?;
-        let mut choices = AHashMap::with_capacity(schema_choices.len());
-        let mut repeat_choices_vec: Vec<(ChoiceKey, ChoiceKey)> = Vec::new();
-        let mut first = true;
+        let choices = PyDict::new(py);
         let mut tags_repr = String::with_capacity(50);
         let mut descr = String::with_capacity(50);
-
-        for (key, value) in schema_choices {
-            let tag = ChoiceKey::from_py(key)?;
-
-            if let Ok(repeat_tag) = ChoiceKey::from_py(value) {
-                repeat_choices_vec.push((tag, repeat_tag));
-                continue;
-            }
-
-            let validator = build_validator(value, config, definitions)?;
-            let tag_repr = tag.repr();
+        let mut first = true;
+        let mut discriminators = Vec::with_capacity(choices.len());
+        let schema_choices: &PyDict = schema.get_as_req(intern!(py, "choices"))?;
+        let mut lookup_map = Vec::with_capacity(choices.len());
+        for (choice_key, choice_schema) in schema_choices.iter() {
+            discriminators.push(choice_key);
+            let validator = build_validator(choice_schema, config, definitions)?;
+            let tag_repr = choice_key.repr()?.to_string();
             if first {
                 first = false;
                 write!(tags_repr, "{tag_repr}").unwrap();
@@ -312,32 +260,10 @@ impl BuildValidator for TaggedUnionValidator {
                 // no spaces in get_name() output to make loc easy to read
                 write!(descr, ",{}", validator.get_name()).unwrap();
             }
-            choices.insert(tag, validator);
+            lookup_map.push((choice_key, validator));
         }
-        let repeat_choices = if repeat_choices_vec.is_empty() {
-            None
-        } else {
-            let mut wrong_values = Vec::with_capacity(repeat_choices_vec.len());
-            let mut repeat_choices = AHashMap::with_capacity(repeat_choices_vec.len());
-            for (tag, repeat_tag) in repeat_choices_vec {
-                match choices.get(&repeat_tag) {
-                    Some(validator) => {
-                        let tag_repr = tag.repr();
-                        write!(tags_repr, ", {tag_repr}").unwrap();
-                        write!(descr, ",{}", validator.get_name()).unwrap();
-                        repeat_choices.insert(tag, repeat_tag);
-                    }
-                    None => wrong_values.push(format!("`{repeat_tag}`")),
-                }
-            }
-            if !wrong_values.is_empty() {
-                return py_err!(
-                    "String values in choices don't match any keys: {}",
-                    wrong_values.join(", ")
-                );
-            }
-            Some(repeat_choices)
-        };
+
+        let lookup = LiteralLookup::new(py, lookup_map.into_iter())?;
 
         let key = intern!(py, "from_attributes");
         let from_attributes = schema_or_config(schema, config, key, key)?.unwrap_or(true);
@@ -348,9 +274,8 @@ impl BuildValidator for TaggedUnionValidator {
         };
 
         Ok(Self {
-            choices,
-            repeat_choices,
             discriminator,
+            lookup,
             from_attributes,
             strict: is_strict(schema, config)?,
             custom_error: CustomError::build(schema, config, definitions)?,
@@ -379,44 +304,33 @@ impl Validator for TaggedUnionValidator {
                         // errors when getting attributes which should be "raised"
                         match lookup_key.$get_method($( $dict ),+)? {
                             Some((_, value)) => {
-                                if let Ok(int) = value.validate_int(self.strict) {
-                                    Ok(ChoiceKey::Int(int))
-                                } else {
-                                    Ok(ChoiceKey::Str(value.validate_str(self.strict)?.as_cow()?.as_ref().to_string()))
-                                }
+                                Ok(value.to_object(py).into_ref(py))
                             }
                             None => Err(self.tag_not_found(input)),
                         }
                     }};
                 }
-                let dict = input.validate_model_fields(self.strict, self.from_attributes)?;
+                let from_attributes = extra.from_attributes.unwrap_or(self.from_attributes);
+                let dict = input.validate_model_fields(self.strict, from_attributes)?;
                 let tag = match dict {
                     GenericMapping::PyDict(dict) => find_validator!(py_get_dict_item, dict),
                     GenericMapping::PyGetAttr(obj, kwargs) => find_validator!(py_get_attr, obj, kwargs),
                     GenericMapping::PyMapping(mapping) => find_validator!(py_get_mapping_item, mapping),
                     GenericMapping::JsonObject(mapping) => find_validator!(json_get, mapping),
                 }?;
-                self.find_call_validator(py, &tag, input, extra, definitions, recursion_guard)
+                self.find_call_validator(py, tag, input, extra, definitions, recursion_guard)
             }
             Discriminator::Function(ref func) => {
                 let tag = func.call1(py, (input.to_object(py),))?;
                 if tag.is_none(py) {
                     Err(self.tag_not_found(input))
                 } else {
-                    let tag: &PyAny = tag.downcast(py)?;
-                    self.find_call_validator(
-                        py,
-                        &(ChoiceKey::from_py(tag)?),
-                        input,
-                        extra,
-                        definitions,
-                        recursion_guard,
-                    )
+                    self.find_call_validator(py, tag.into_ref(py), input, extra, definitions, recursion_guard)
                 }
             }
             Discriminator::SelfSchema => self.find_call_validator(
                 py,
-                &ChoiceKey::Str(self.self_schema_tag(py, input)?.into_owned()),
+                self.self_schema_tag(py, input)?.as_ref(),
                 input,
                 extra,
                 definitions,
@@ -430,8 +344,9 @@ impl Validator for TaggedUnionValidator {
         definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
         ultra_strict: bool,
     ) -> bool {
-        self.choices
-            .values()
+        self.lookup
+            .values
+            .iter()
             .any(|v| v.different_strict_behavior(definitions, ultra_strict))
     }
 
@@ -440,9 +355,10 @@ impl Validator for TaggedUnionValidator {
     }
 
     fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.choices
+        self.lookup
+            .values
             .iter_mut()
-            .try_for_each(|(_, validator)| validator.complete(definitions))
+            .try_for_each(|validator| validator.complete(definitions))
     }
 }
 
@@ -451,7 +367,7 @@ impl TaggedUnionValidator {
         &'s self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-    ) -> ValResult<'data, Cow<'data, str>> {
+    ) -> ValResult<'data, &'data PyString> {
         let dict = input.strict_dict()?;
         let either_tag = match dict {
             GenericMapping::PyDict(dict) => match dict.get_item(intern!(py, "type")) {
@@ -474,46 +390,38 @@ impl TaggedUnionValidator {
             if tag == "function" {
                 let mode = mode.ok_or_else(|| self.tag_not_found(input))?;
                 match mode.as_cow()?.as_ref() {
-                    "plain" => Ok(Cow::Borrowed("function-plain")),
-                    "wrap" => Ok(Cow::Borrowed("function-wrap")),
-                    _ => Ok(Cow::Borrowed("function")),
+                    "plain" => Ok(intern!(py, "function-plain")),
+                    "wrap" => Ok(intern!(py, "function-wrap")),
+                    _ => Ok(intern!(py, "function")),
                 }
             } else {
                 // tag == "tuple"
                 if let Some(mode) = mode {
                     if mode.as_cow()?.as_ref() == "positional" {
-                        return Ok(Cow::Borrowed("tuple-positional"));
+                        return Ok(intern!(py, "tuple-positional"));
                     }
                 }
-                Ok(Cow::Borrowed("tuple-variable"))
+                Ok(intern!(py, "tuple-variable"))
             }
         } else {
-            Ok(Cow::Owned(tag.to_string()))
+            Ok(PyString::new(py, tag))
         }
     }
 
     fn find_call_validator<'s, 'data>(
         &'s self,
         py: Python<'data>,
-        tag: &ChoiceKey,
+        tag: &'data PyAny,
         input: &'data impl Input<'data>,
         extra: &Extra,
         definitions: &'data Definitions<CombinedValidator>,
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
-        if let Some(validator) = self.choices.get(tag) {
+        if let Ok(Some((tag, validator))) = self.lookup.validate(py, tag) {
             return match validator.validate(py, input, extra, definitions, recursion_guard) {
                 Ok(res) => Ok(res),
-                Err(err) => Err(err.with_outer_location(tag.into())),
+                Err(err) => Err(err.with_outer_location(LocItem::try_from(tag.to_object(py).into_ref(py))?)),
             };
-        } else if let Some(ref repeat_choices) = self.repeat_choices {
-            if let Some(choice_tag) = repeat_choices.get(tag) {
-                let validator = &self.choices[choice_tag];
-                return match validator.validate(py, input, extra, definitions, recursion_guard) {
-                    Ok(res) => Ok(res),
-                    Err(err) => Err(err.with_outer_location(tag.into())),
-                };
-            }
         }
         match self.custom_error {
             Some(ref custom_error) => Err(custom_error.as_val_error(input)),
