@@ -41,7 +41,7 @@ class TestBenchmarkSimpleModel:
     @pytest.fixture(scope='class')
     def core_validator_fs(self):
         class CoreModel:
-            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+            __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         return SchemaValidator(
             {
@@ -101,7 +101,7 @@ class TestModelLarge:
     @pytest.fixture(scope='class')
     def core_model_validator(self):
         class CoreModel:
-            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+            __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         return SchemaValidator(
             {
@@ -193,7 +193,7 @@ def test_small_class_core_dict(benchmark):
 def test_small_class_core_model(benchmark):
     class MyCoreModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         # these are here just as decoration
         name: str
         age: int
@@ -344,7 +344,7 @@ def test_definition_model_pyd(definition_model_data, benchmark):
 def test_definition_model_core(definition_model_data, benchmark):
     class CoreBranch:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     v = SchemaValidator(
         {
@@ -678,7 +678,7 @@ def test_many_models_core_dict(benchmark):
 @pytest.mark.benchmark(group='List[SimpleMode]')
 def test_many_models_core_model(benchmark):
     class MyCoreModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     v = SchemaValidator(
         {
@@ -745,7 +745,7 @@ class TestBenchmarkDateTime:
     @pytest.fixture(scope='class')
     def core_validator(self):
         class CoreModel:
-            __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+            __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         return SchemaValidator(
             {
@@ -1234,7 +1234,7 @@ def test_definition_out_of_tree(benchmark):
 @pytest.mark.benchmark(group='model_instance')
 def test_model_instance(benchmark):
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def __init__(self, **d):
             self.__dict__ = d
@@ -1271,7 +1271,7 @@ def test_model_instance_abc(benchmark):
             return hasattr(instance, '__pydantic_validator__') and super().__instancecheck__(instance)
 
     class BaseModel(metaclass=MyMeta):
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         __pydantic_validator__ = True
 
         def __init__(self, **d):
@@ -1314,6 +1314,7 @@ class SomeStrEnum(str, Enum):
 LARGE_STR_PREFIX = 'a' * 50
 
 
+@pytest.mark.benchmark(group='validate_literal')
 @pytest.mark.parametrize(
     'allowed_values,input,expected_val_res',
     [
@@ -1380,3 +1381,72 @@ def test_v1_root_model(benchmark):
     input_data = list(range(100))
 
     benchmark(MyModel.parse_obj, input_data)
+
+
+@pytest.mark.benchmark(group='strict_int')
+def test_strict_int(benchmark):
+    v = SchemaValidator(core_schema.int_schema(strict=True))
+
+    benchmark(v.validate_python, 42)
+
+
+@pytest.mark.benchmark(group='int_range')
+def test_int_range(benchmark):
+    v = SchemaValidator(core_schema.int_schema(gt=0, lt=100))
+
+    assert v.validate_python(42) == 42
+    with pytest.raises(ValidationError, match='Input should be greater than 0'):
+        v.validate_python(0)
+
+    benchmark(v.validate_python, 42)
+
+
+@pytest.mark.benchmark(group='int_range')
+def test_int_range_json(benchmark):
+    v = SchemaValidator(core_schema.int_schema(gt=0, lt=100))
+
+    assert v.validate_json('42') == 42
+    with pytest.raises(ValidationError, match='Input should be greater than 0'):
+        v.validate_python('0')
+
+    benchmark(v.validate_json, '42')
+
+
+@pytest.mark.benchmark(group='tagged_union_ints')
+def test_tagged_union_int_keys_python(benchmark):
+    inner_schema = core_schema.typed_dict_schema(
+        {
+            'x': core_schema.typed_dict_field(core_schema.int_schema()),
+            'y': core_schema.typed_dict_field(core_schema.int_schema()),
+        }
+    )
+    v = SchemaValidator(core_schema.tagged_union_schema({x: inner_schema for x in range(1000)}, discriminator='x'))
+
+    payload = {'x': 999, 'y': '1'}
+    assert v.validate_python(payload) == {'x': 999, 'y': 1}
+    with pytest.raises(
+        ValidationError, match="Input tag '1001' found using 'x' does not match any of the expected tags"
+    ):
+        v.validate_python({'x': 1001, 'y': '1'})
+
+    benchmark(v.validate_python, payload)
+
+
+@pytest.mark.benchmark(group='tagged_union_ints')
+def test_tagged_union_int_keys_json(benchmark):
+    inner_schema = core_schema.typed_dict_schema(
+        {
+            'x': core_schema.typed_dict_field(core_schema.int_schema()),
+            'y': core_schema.typed_dict_field(core_schema.int_schema()),
+        }
+    )
+    v = SchemaValidator(core_schema.tagged_union_schema({x: inner_schema for x in range(1000)}, discriminator='x'))
+
+    payload = '{"x": 999, "y": "1"}'
+    assert v.validate_json(payload) == {'x': 999, 'y': 1}
+    with pytest.raises(
+        ValidationError, match="Input tag '1001' found using 'x' does not match any of the expected tags"
+    ):
+        v.validate_json('{"x": 1001, "y": "1"}')
+
+    benchmark(v.validate_json, payload)
