@@ -2,7 +2,8 @@ import pickle
 
 import pytest
 
-from pydantic_core import SchemaError, SchemaValidator
+from pydantic_core import SchemaError, SchemaValidator, __version__
+from pydantic_core import core_schema as cs
 
 
 def test_build_error_type():
@@ -36,7 +37,8 @@ def test_schema_wrong_type():
         SchemaValidator(1)
     assert str(exc_info.value) == (
         'Invalid Schema:\n  Input should be a valid dictionary or instance to'
-        ' extract fields from [type=dict_attributes_type, input_value=1, input_type=int]'
+        ' extract fields from [type=dict_attributes_type, input_value=1, input_type=int]\n'
+        f'    For further information visit https://errors.pydantic.dev/{__version__}/v/dict_attributes_type'
     )
     assert exc_info.value.errors() == [
         {
@@ -97,3 +99,42 @@ def test_try_self_schema_discriminator():
     """Trying to use self-schema when it shouldn't be used"""
     v = SchemaValidator({'type': 'tagged-union', 'choices': {'int': {'type': 'int'}}, 'discriminator': 'self-schema'})
     assert 'discriminator: LookupKey' in repr(v)
+
+
+def test_build_recursive_schema_from_defs() -> None:
+    """
+    Validate a schema representing mutually recursive models, analogous to the following JSON schema:
+
+    ```json
+    {
+        "$schema": "https://json-schema.org/draft/2019-09/schema",
+        "oneOf": [{"$ref": "#/$defs/a"}],
+        "$defs": {
+            "a": {
+                "type": "object",
+                "properties": {"b": {"type": "array", "items": {"$ref": "#/$defs/a"}}},
+                "required": ["b"],
+            },
+            "b": {
+                "type": "object",
+                "properties": {"a": {"type": "array", "items": {"$ref": "#/$defs/b"}}},
+                "required": ["a"],
+            },
+        },
+    }
+    ```
+    """
+
+    s = cs.definitions_schema(
+        cs.definition_reference_schema(schema_ref='a'),
+        [
+            cs.typed_dict_schema(
+                {'b': cs.typed_dict_field(cs.list_schema(cs.definition_reference_schema('b')))}, ref='a'
+            ),
+            cs.typed_dict_schema(
+                {'a': cs.typed_dict_field(cs.list_schema(cs.definition_reference_schema('a')))}, ref='b'
+            ),
+        ],
+    )
+
+    SchemaValidator(s)
