@@ -224,6 +224,7 @@ all_errors = [
     ('bool_parsing', 'Input should be a valid boolean, unable to interpret input', None),
     ('int_type', 'Input should be a valid integer', None),
     ('int_parsing', 'Input should be a valid integer, unable to parse string as an integer', None),
+    ('int_parsing_size', 'Unable to parse input string as an integer, exceeded maximum size', None),
     ('int_from_float', 'Input should be a valid integer, got a number with a fractional part', None),
     ('multiple_of', 'Input should be a multiple of 42.1', {'multiple_of': 42.1}),
     ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
@@ -250,10 +251,11 @@ all_errors = [
     ('datetime_type', 'Input should be a valid datetime', None),
     ('datetime_parsing', 'Input should be a valid datetime, foobar', {'error': 'foobar'}),
     ('datetime_object_invalid', 'Invalid datetime object, got foobar', {'error': 'foobar'}),
-    ('datetime_past', 'Datetime should be in the past', None),
-    ('datetime_future', 'Datetime should be in the future', None),
-    ('datetime_aware', 'Datetime should have timezone info', None),
-    ('datetime_naive', 'Datetime should not have timezone info', None),
+    ('datetime_past', 'Input should be in the past', None),
+    ('datetime_future', 'Input should be in the future', None),
+    ('timezone_naive', 'Input should not have timezone info', None),
+    ('timezone_aware', 'Input should have timezone info', None),
+    ('timezone_offset', 'Timezone offset of 0 required, got 60', {'tz_expected': 0, 'tz_actual': 60}),
     ('time_delta_type', 'Input should be a valid timedelta', None),
     ('time_delta_parsing', 'Input should be a valid timedelta, foobar', {'error': 'foobar'}),
     ('frozen_set_type', 'Input should be a valid frozenset', None),
@@ -565,6 +567,19 @@ def test_raise_validation_error():
         raise ValidationError.from_exception_data('Foobar', [{'type': 'greater_than', 'loc': ('a', 2), 'input': 4}])
 
 
+@pytest.mark.parametrize(
+    'hide_input_in_errors,input_str',
+    ((False, 'type=greater_than, input_value=4, input_type=int'), (True, 'type=greater_than')),
+)
+def test_raise_validation_error_hide_input(hide_input_in_errors, input_str):
+    with pytest.raises(ValidationError, match=re.escape(f'Input should be greater than 5 [{input_str}]')):
+        raise ValidationError.from_exception_data(
+            'Foobar',
+            [{'type': 'greater_than', 'loc': ('a', 2), 'input': 4, 'ctx': {'gt': 5}}],
+            hide_input=hide_input_in_errors,
+        )
+
+
 def test_raise_validation_error_json():
     with pytest.raises(ValidationError) as exc_info:
         raise ValidationError.from_exception_data('Foobar', [{'type': 'none_required', 'loc': [-42], 'input': 'x'}])
