@@ -2,7 +2,7 @@ import re
 
 import pytest
 
-from pydantic_core import SchemaValidator, ValidationError, core_schema
+from pydantic_core import SchemaValidator, ValidationError, __version__, core_schema
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -62,9 +62,10 @@ def test_bool_error():
     assert str(exc_info.value) == (
         '1 validation error for bool\n'
         '  Input should be a valid boolean, '
-        "unable to interpret input [type=bool_parsing, input_value='wrong', input_type=str]"
+        "unable to interpret input [type=bool_parsing, input_value='wrong', input_type=str]\n"
+        f"    For further information visit https://errors.pydantic.dev/{__version__}/v/bool_parsing"
     )
-    assert exc_info.value.errors() == [
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'bool_parsing',
             'loc': (),
@@ -76,9 +77,9 @@ def test_bool_error():
 
 def test_bool_repr():
     v = SchemaValidator({'type': 'bool'})
-    assert plain_repr(v) == 'SchemaValidator(title="bool",validator=Bool(BoolValidator{strict:false}),slots=[])'
+    assert plain_repr(v) == 'SchemaValidator(title="bool",validator=Bool(BoolValidator{strict:false}),definitions=[])'
     v = SchemaValidator({'type': 'bool', 'strict': True})
-    assert plain_repr(v) == 'SchemaValidator(title="bool",validator=Bool(BoolValidator{strict:true}),slots=[])'
+    assert plain_repr(v) == 'SchemaValidator(title="bool",validator=Bool(BoolValidator{strict:true}),definitions=[])'
 
 
 def test_bool_key(py_and_json: PyAndJson):
