@@ -1,11 +1,10 @@
-import re
 from decimal import Decimal
 
 import pytest
 
 from pydantic_core import SchemaError, SchemaValidator, ValidationError
 
-from ..conftest import PyAndJson, plain_repr
+from ..conftest import PyAndJson
 
 
 def test_chain():
@@ -47,8 +46,8 @@ def test_chain_error():
 
     with pytest.raises(ValidationError) as exc_info:
         validator.validate_python('abc')
-    # insert_assert(exc_info.value.errors())
-    assert exc_info.value.errors() == [
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
         {
             'type': 'int_parsing',
             'loc': (),
@@ -119,39 +118,3 @@ def test_chain_one():
     )
     assert validator.validate_python('input') == 'input-1'
     assert validator.title == 'function-plain[<lambda>()]'
-
-
-def test_ask():
-    class MyModel:
-        __slots__ = '__dict__', '__fields_set__'
-
-    calls = []
-
-    def f(input_value, info):
-        calls.append(input_value)
-        return input_value
-
-    v = SchemaValidator(
-        {
-            'type': 'model',
-            'cls': MyModel,
-            'schema': {
-                'type': 'chain',
-                'steps': [
-                    {
-                        'type': 'typed-dict',
-                        'return_fields_set': True,
-                        'fields': {'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-                    },
-                    {'type': 'function-plain', 'function': {'type': 'general', 'function': f}},
-                ],
-            },
-        }
-    )
-    assert re.search('expect_fields_set:(true|false)', plain_repr(v)).group(1) == 'true'
-    m = v.validate_python({'field_a': 'abc'})
-    assert isinstance(m, MyModel)
-    assert m.field_a == 'abc'
-    assert m.__fields_set__ == {'field_a'}
-    # insert_assert(calls)
-    assert calls == [({'field_a': 'abc'}, {'field_a'})]
