@@ -44,10 +44,10 @@ def test_no_copy():
     'input_value,expected',
     [
         ([1, 2.5, '3'], {1, 2.5, '3'}),
-        ('foo', Err('Input should be a valid frozenset')),
-        (1, Err('Input should be a valid frozenset')),
-        (1.0, Err('Input should be a valid frozenset')),
-        (False, Err('Input should be a valid frozenset')),
+        ('foo', Err("[type=frozen_set_type, input_value='foo', input_type=str]")),
+        (1, Err('[type=frozen_set_type, input_value=1, input_type=int]')),
+        (1.0, Err('[type=frozen_set_type, input_value=1.0, input_type=float]')),
+        (False, Err('[type=frozen_set_type, input_value=False, input_type=bool]')),
     ],
 )
 def test_frozenset_no_validators_both(py_and_json: PyAndJson, input_value, expected):
@@ -237,7 +237,7 @@ def test_union_frozenset_int_frozenset_str(input_value, expected):
 
 def test_frozenset_as_dict_keys(py_and_json: PyAndJson):
     v = py_and_json({'type': 'dict', 'keys_schema': {'type': 'frozenset'}, 'values_schema': {'type': 'int'}})
-    with pytest.raises(ValidationError, match=re.escape('Input should be a valid frozenset')):
+    with pytest.raises(ValidationError, match=re.escape("[type=int_parsing, input_value='bar', input_type=str]")):
         v.validate_test({'foo': 'bar'})
 
 
