@@ -10,7 +10,7 @@ from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_sc
 def test_model_class():
     class MyModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
 
@@ -34,25 +34,19 @@ def test_model_class():
     assert m.__pydantic_fields_set__ == {'field_a', 'field_b'}
     assert m.__dict__ == {'field_a': 'test', 'field_b': 12}
 
-    with pytest.raises(ValidationError, match='Input should be an instance of MyModel') as exc_info:
-        v.validate_python({'field_a': 'test', 'field_b': 12}, strict=True)
-
-    # insert_assert(exc_info.value.errors(include_url=False))
-    assert exc_info.value.errors(include_url=False) == [
-        {
-            'type': 'model_class_type',
-            'loc': (),
-            'msg': 'Input should be an instance of MyModel',
-            'input': {'field_a': 'test', 'field_b': 12},
-            'ctx': {'class_name': 'MyModel'},
-        }
-    ]
+    m2 = v.validate_python({'field_a': 'test', 'field_b': 12}, strict=True)
+    assert isinstance(m2, MyModel)
+    assert m2.field_a == 'test'
+    assert m2.field_b == 12
+    assert m2.__pydantic_extra__ is None
+    assert m2.__pydantic_fields_set__ == {'field_a', 'field_b'}
+    assert m2.__dict__ == {'field_a': 'test', 'field_b': 12}
 
 
 def test_model_class_extra():
     class MyModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
 
@@ -221,7 +215,7 @@ def test_model_class_root_validator_after():
 @pytest.mark.parametrize('mode', ['before', 'after', 'wrap'])
 def test_function_ask(mode):
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     def f(input_value, info):
         return input_value
@@ -244,7 +238,7 @@ def test_function_ask(mode):
 
 def test_function_plain_ask():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     def f(input_value):
         return input_value, {1: 2}, {'field_a'}
@@ -265,7 +259,7 @@ def test_function_plain_ask():
 
 def test_union_sub_schema():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     v = SchemaValidator(
         {
@@ -330,6 +324,7 @@ def test_tagged_union_sub_schema():
         'bar': 123,
         '__pydantic_fields_set__': {'foo', 'bar'},
         '__pydantic_extra__': None,
+        '__pydantic_private__': None,
     }
 
     m = v.validate_python({'foo': 'banana', 'spam': [1, 2, 3]})
@@ -340,6 +335,7 @@ def test_tagged_union_sub_schema():
         'spam': [1, 2, 3],
         '__pydantic_fields_set__': {'spam', 'foo'},
         '__pydantic_extra__': None,
+        '__pydantic_private__': None,
     }
 
 
@@ -354,7 +350,7 @@ def test_bad_sub_schema():
 
 def test_model_class_function_after():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     def f(input_value, info):
         input_value[0]['x'] = 'y'
@@ -396,7 +392,7 @@ def test_model_class_not_type():
 
 def test_model_class_instance_direct():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
         def __init__(self):
@@ -427,7 +423,7 @@ def test_model_class_instance_subclass():
     post_init_calls = []
 
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
         def __init__(self):
@@ -474,7 +470,7 @@ def test_model_class_instance_subclass_revalidate():
     post_init_calls = []
 
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
         def __init__(self):
@@ -554,18 +550,10 @@ def test_model_class_strict():
     assert m.field_a == 'init_a'
     # note that since dict validation was not run here, there has been no check this is an int
     assert m.field_b == 'init_b'
-    with pytest.raises(ValidationError, match='^1 validation error for MyModel\n') as exc_info:
-        v.validate_python({'field_a': 'test', 'field_b': 12})
-    assert exc_info.value.errors(include_url=False) == [
-        {
-            'type': 'model_class_type',
-            'loc': (),
-            'msg': 'Input should be an instance of MyModel',
-            'input': {'field_a': 'test', 'field_b': 12},
-            'ctx': {'class_name': 'MyModel'},
-        }
-    ]
-    assert str(exc_info.value).startswith('1 validation error for MyModel\n')
+    m3 = v.validate_python({'field_a': 'test', 'field_b': 12})
+    assert isinstance(m3, MyModel)
+    assert m3.field_a == 'test'
+    assert m3.field_b == 12
 
     class MySubModel(MyModel):
         field_c: str
@@ -582,7 +570,7 @@ def test_model_class_strict():
 
 def test_model_class_strict_json():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
         field_c: int
@@ -627,7 +615,7 @@ def test_internal_error():
 
 def test_revalidate_always():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def __init__(self, a, b, fields_set):
             self.field_a = a
@@ -643,7 +631,6 @@ def test_revalidate_always():
             'revalidate_instances': 'always',
             'schema': {
                 'type': 'model-fields',
-                'from_attributes': True,
                 'fields': {
                     'field_a': {'type': 'model-field', 'schema': {'type': 'str'}},
                     'field_b': {'type': 'model-field', 'schema': {'type': 'int'}},
@@ -684,7 +671,7 @@ def test_revalidate_always():
 
 def test_revalidate_subclass_instances():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def __init__(self):
             self.field_a = 'init_a'
@@ -702,7 +689,6 @@ def test_revalidate_subclass_instances():
             'revalidate_instances': 'subclass-instances',
             'schema': {
                 'type': 'model-fields',
-                'from_attributes': True,
                 'fields': {
                     'field_a': {'type': 'model-field', 'schema': {'type': 'str'}},
                     'field_b': {'type': 'model-field', 'schema': {'type': 'int'}},
@@ -734,7 +720,7 @@ def test_revalidate_subclass_instances():
 
 def test_revalidate_extra():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def __init__(self, **kwargs):
             self.__dict__.update(kwargs)
@@ -745,7 +731,6 @@ def test_revalidate_extra():
             'cls': MyModel,
             'schema': {
                 'type': 'model-fields',
-                'from_attributes': True,
                 'extra_behavior': 'allow',
                 'fields': {
                     'field_a': {'type': 'model-field', 'schema': {'type': 'str'}},
@@ -777,7 +762,7 @@ def test_post_init():
     call_count = 0
 
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
 
@@ -814,7 +799,7 @@ def test_revalidate_post_init():
     call_count = 0
 
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def call_me_maybe(self, context):
             nonlocal call_count
@@ -828,7 +813,6 @@ def test_revalidate_post_init():
             'post_init': 'call_me_maybe',
             'schema': {
                 'type': 'model-fields',
-                'from_attributes': True,
                 'fields': {
                     'field_a': {'type': 'model-field', 'schema': {'type': 'str'}},
                     'field_b': {'type': 'model-field', 'schema': {'type': 'int'}},
@@ -861,7 +845,7 @@ def test_revalidate_post_init():
 
 def test_post_init_validation_error():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
         def call_me_maybe(self, context, **kwargs):
@@ -898,7 +882,7 @@ def test_post_init_validation_error():
 
 def test_post_init_internal_error():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
         def wrong_signature(self):
@@ -921,7 +905,7 @@ def test_post_init_internal_error():
 
 def test_post_init_mutate():
     class MyModel:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
 
@@ -954,7 +938,7 @@ def test_post_init_mutate():
 def test_validate_assignment():
     class MyModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
 
@@ -983,11 +967,14 @@ def test_validate_assignment():
     assert m.field_b == 321
     assert m.__pydantic_fields_set__ == {'field_a', 'field_b'}
 
+    v.validate_assignment(m, 'field_b', '322', from_attributes=True)
+    assert m.field_b == 322
+
 
 def test_validate_assignment_function():
     class MyModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
         field_b: int
         field_c: int
@@ -1065,7 +1052,7 @@ def test_validate_assignment_no_fields_set():
 
 def test_frozen():
     class MyModel:
-        __slots__ = {'__dict__', '__pydantic_extra__', '__pydantic_fields_set__'}
+        __slots__ = {'__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'}
 
     v = SchemaValidator(
         core_schema.model_schema(
@@ -1121,7 +1108,7 @@ def test_validate_assignment_model_validator_function(function_schema: Any, call
     """
 
     class Model:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     calls: List[Any] = []
 
