@@ -5,7 +5,7 @@ from pydantic_core import CoreConfig, SchemaValidator, core_schema
 
 class MyModel:
     # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-    __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+    __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
     field_a: str
     field_b: int
 
@@ -41,7 +41,7 @@ def test_model_init():
 def test_model_init_nested():
     class MyModel:
         # this is not required, but it avoids `__pydantic_fields_set__` being included in `__dict__`
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     v = SchemaValidator(
         {
@@ -259,7 +259,7 @@ def test_model_custom_init_nested():
     calls = []
 
     class ModelInner:
-        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         a: int
         b: int
 
@@ -315,7 +315,7 @@ def test_model_custom_init_extra():
     calls = []
 
     class ModelInner:
-        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         a: int
         b: int
 
@@ -340,7 +340,7 @@ def test_model_custom_init_extra():
     ModelInner.__pydantic_validator__ = SchemaValidator(inner_schema)
 
     class ModelOuter:
-        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         a: int
         b: ModelInner
 
@@ -385,7 +385,7 @@ def test_model_custom_init_revalidate():
     calls = []
 
     class Model:
-        __slots__ = '__dict__', '__pydantic_extra__', '__pydantic_fields_set__'
+        __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
         def __init__(self, **kwargs):
             calls.append(repr(kwargs))
